@@ -2,12 +2,15 @@
 
 import gi
 gi.require_version('Gtk', '3.0')
+gi.require_version('XApp', '1.0')
 import sys
-sys.path.append('/usr/share/cinnamon/cinnamon-settings/bin')
+
+import config
+sys.path.append(config.currentPath + "/bin")
 import gettext
 import json
 from JsonSettingsWidgets import *
-from gi.repository import Gtk, Gio
+from gi.repository import Gtk, Gio, XApp
 
 # i18n
 gettext.install("cinnamon", "/usr/share/locale")
@@ -119,10 +122,10 @@ class MainWindow(object):
             self.xlet_meta = json.loads(raw_data)
         else:
             print("Could not find %s metadata for uuid %s - are you sure it's installed correctly?" % (self.type, self.uuid))
-            self.quit()
+            quit()
 
     def build_window(self):
-        self.window = Gtk.Window()
+        self.window = XApp.GtkWindow()
         self.window.set_default_size(800, 600)
         main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.window.add(main_box)
@@ -242,10 +245,10 @@ class MainWindow(object):
             first_key = next(iter(settings_map.values()))
 
             try:
-                for setting in settings_map.keys():
+                for setting in settings_map:
                     if setting == "__md5__":
                         continue
-                    for key in settings_map[setting].keys():
+                    for key in settings_map[setting]:
                         if key in ("description", "tooltip", "units"):
                             try:
                                 settings_map[setting][key] = translate(self.uuid, settings_map[setting][key])
@@ -254,11 +257,15 @@ class MainWindow(object):
                         elif key in "options":
                             new_opt_data = collections.OrderedDict()
                             opt_data = settings_map[setting][key]
-                            for option in opt_data.keys():
+                            for option in opt_data:
                                 if opt_data[option] == "custom":
                                     continue
                                 new_opt_data[translate(self.uuid, option)] = opt_data[option]
                             settings_map[setting][key] = new_opt_data
+                        elif key in "columns":
+                            columns_data = settings_map[setting][key]
+                            for column in columns_data:
+                                column["title"] = translate(self.uuid, column["title"])
             finally:
                 # if a layout is not explicitly defined, generate the settings
                 # widgets based on the order they occur
@@ -292,18 +299,27 @@ class MainWindow(object):
             page_stack.add_titled(page, page_key, translate(self.uuid, page_def["title"]))
             for section_key in page_def["sections"]:
                 section_def = layout[section_key]
-                section = page.add_section(translate(self.uuid, section_def["title"]))
+                if 'dependency' in section_def:
+                    revealer = JSONSettingsRevealer(info['settings'], section_def['dependency'])
+                    section = page.add_reveal_section(translate(self.uuid, section_def["title"]), revealer=revealer)
+                else:
+                    section = page.add_section(translate(self.uuid, section_def["title"]))
                 for key in section_def["keys"]:
                     item = settings_map[key]
                     settings_type = item["type"]
                     if settings_type == "button":
                         widget = XLETSettingsButton(item, self.uuid, info["id"])
-                        section.add_row(widget)
                     elif settings_type == "label":
                         widget = Text(translate(self.uuid, item["description"]))
-                        section.add_row(widget)
                     elif settings_type in XLET_SETTINGS_WIDGETS:
                         widget = globals()[XLET_SETTINGS_WIDGETS[settings_type]](key, info["settings"], item)
+                    else:
+                        continue
+
+                    if 'dependency' in item:
+                        revealer = JSONSettingsRevealer(info['settings'], item['dependency'])
+                        section.add_reveal_row(widget, revealer=revealer)
+                    else:
                         section.add_row(widget)
 
     def build_from_order(self, settings_map, info, box, first_key):
@@ -317,20 +333,33 @@ class MainWindow(object):
         for key, item in settings_map.items():
             if key == "__md5__":
                 continue
-            if "type" in item.keys():
+            if "type" in item:
                 settings_type = item["type"]
                 if settings_type in ("header", "section"):
-                    section = page.add_section(translate(self.uuid, item["description"]))
-                elif settings_type == "button":
+                    if 'dependency' in item:
+                        revealer = JSONSettingsRevealer(info['settings'], item['dependency'])
+                        section = page.add_reveal_section(translate(self.uuid, item["description"]), revealer=revealer)
+                    else:
+                        section = page.add_section(translate(self.uuid, item["description"]))
+                    continue
+
+                if settings_type == "button":
                     widget = XLETSettingsButton(item, self.uuid, info["id"])
-                    section.add_row(widget)
                 elif settings_type == "label":
                     widget = Text(translate(self.uuid, item["description"]))
-                    section.add_row(widget)
                 elif settings_type in XLET_SETTINGS_WIDGETS:
                     widget = globals()[XLET_SETTINGS_WIDGETS[settings_type]](key, info["settings"], item)
+                else:
+                    continue
+
+                if 'dependency' in item:
+                    revealer = JSONSettingsRevealer(info['settings'], item['dependency'])
+                    section.add_reveal_row(widget, revealer=revealer)
+                else:
                     section.add_row(widget)
 
+
+
     def notify_dbus(self, handler, key, value):
         proxy.updateSetting('(ssss)', self.uuid, handler.instance_id, key, json.dumps(value))
 
@@ -367,8 +396,8 @@ class MainWindow(object):
         dialog = Gtk.FileChooserDialog(_("Select or enter file to export to"),
                                        None,
                                        Gtk.FileChooserAction.SAVE,
-                                      (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                                       Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT))
+                                       (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                                        Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT))
         dialog.set_do_overwrite_confirmation(True)
         filter_text = Gtk.FileFilter()
         filter_text.add_pattern("*.json")
@@ -389,8 +418,8 @@ class MainWindow(object):
         dialog = Gtk.FileChooserDialog(_("Select a JSON file to import"),
                                        None,
                                        Gtk.FileChooserAction.OPEN,
-                                      (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                                       Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+                                       (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                                        Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
         filter_text = Gtk.FileFilter()
         filter_text.add_pattern("*.json")
         filter_text.set_name(_("JSON files"))
